﻿// Class definition for the JavaScript object for the client side of the TrackBar control.
function TrackBar(clientID, minimum, maximum, smallChange, barPixelWidth) // constructor parameters
{
    // In this pattern of JavaScript classes, the methods follow here, the 'constructor' is
    // at the bottom of the class.

    // Slider is being dragged
    TrackBar.prototype.DragMouseMove = function(event)
    {
        // The location to which the user is attempting to drag the slider is calculated
        var eventCoords = CoordsFromEvent(event);

        // The 'drag adjusted' new desired slider absolute X position.
        var calcX = this.originalSliderX + (eventCoords.x - this.originalMouseX);
        
        // This value is capped to lie within the valid range
        if (calcX < this.sliderLeft)
        {
            calcX = this.sliderLeft;
        }
        else if (calcX > this.sliderLeft + this.actualBarPixelWidth)
        {
            calcX = this.sliderLeft + this.actualBarPixelWidth;
        }
        
        // The proportion 0 -> 1 that the slider is selected on
        var propX = (calcX - this.sliderLeft) / this.actualBarPixelWidth;
       
        // Work out position of slider not including rounding to stops
        var value = this.minimum + (this.maximum - this.minimum) * propX;

        // If 'smallChange' is set, restrict slider stops to those values
        if (smallChange != 0)
        {
            // "Quantise" to small change
            value = Math.floor(value / smallChange + 0.5) * smallChange;
        }
        
        // Set text box
        this.valueTextBox.value = value;

        // Use the existing code to correctly position the slider bar
        this.Reposition();

        // Stop the mouse events from being handled by the system
        if ( window.event == null )
	    { // e.g. Firefox
		    event.preventDefault();
	    }
	    else
	    { // e.g. IE
		    window.event.cancelBubble = true;
		    window.event.returnValue = false;
	    }
	    
	    return false;
    }
    

    // Slider has been dropped
    TrackBar.prototype.SliderDropped = function(event)
    {
        // restore the visual appearance of the slider button
        this.slider.src = "slideroff.gif";

        // put the mouse pointer back to a pointer (some browsers)
        document.body.style.cursor = "default";

        // stop mousemove and mouseup events from being captured
        if (document.removeEventListener != null)
        { // e.g. Firefox
	        document.removeEventListener("mousemove", this.mouseMoveDelegate, true);
	        document.removeEventListener("mouseup", this.sliderDroppedDelegate, true);
        }
        else 
        { // e.g. IE
	        document.detachEvent("onmousemove", this.mouseMoveDelegate);
	        document.detachEvent("onmouseup", this.sliderDroppedDelegate);
        }
	    
	    // stop event default
	    if (document.detachEvent == null)
	    { // e.g. Firefox
            event.preventDefault();
	    }
        else 
	    { // e.g. IE
            window.event.cancelBubble = true;
		    window.event.returnValue = false;
	    }
    }
    
    // The slider is being dragged
    TrackBar.prototype.SliderDrag = function(event, id)
    {    
        var rightClicked;

        this.mouseMoveDelegate = MakeDelegate(this, this.DragMouseMove);
        this.sliderDroppedDelegate = MakeDelegate(this, this.SliderDropped);

        if (document.addEventListener != null)
	    { // e.g. Firefox
    
            event.preventDefault();
    
            if (event.button & 2)
	        {
                rightClicked = true;
	        }
            else
            {
                document.addEventListener("mousemove", this.mouseMoveDelegate, true);
		        document.addEventListener("mouseup", this.sliderDroppedDelegate, true);
		        rightClicked = false;
		    }
	    }
	    else
	    { // e.g. IE
	        
	        window.event.cancelBubble = true;
		    window.event.returnValue = false;
		    
	        if (window.event.button & 2)
	        {
	            rightClicked = true;
	        }
            else
            {	 
		        document.attachEvent("onmousemove", this.mouseMoveDelegate);
		        document.attachEvent("onmouseup", this.sliderDroppedDelegate);
		        rightClicked = false;
		    }
	    }

        // Only permit drag with left mouse button
        if (!rightClicked)
        {    
            // change the mouse pointer to a hand (some browsers)
            document.body.style.cursor = "hand";
            var eventCoords = CoordsFromEvent(event);
            this.originalMouseX = eventCoords.x;
            
            position = ElementPosition(this.slider);
            this.originalSliderX = position.x;

            // change visual appearance of the slider button
            this.slider.src = "slideron.gif";
        }    	    
       
	    return true;
    }

    // Position the slider from the text value box
    TrackBar.prototype.Reposition = function()
    {
        // Calculate proportion
        var propX = (this.valueTextBox.value - this.minimum) / this.maximum;

        // Calculate bar position
        var calcX = this.sliderLeft + this.actualBarPixelWidth * propX;

        // This value is capped to lie within the valid range
        if (calcX < this.sliderLeft)
        {
            calcX = this.sliderLeft;
        }
        else if (calcX > this.sliderLeft + this.actualBarPixelWidth)
        {
            calcX = this.sliderLeft + this.actualBarPixelWidth;
        }

        var setSliderLeft = Math.floor(calcX) + "px";
        // Position the slider
        if (this.slider.style.left != setSliderLeft)
        {
            this.slider.style.left = setSliderLeft;
            
            if (this.valueChanged != null)
            {
                this.valueChanged(this);
            }
        }
    }

    // Load event to continue construction of the control
    // (Element positions are not available in the constructor)
    TrackBar.prototype.Load = function(event, id)
    {    
        // Calculate the extents of the slider bar
        this.sliderLeft = ElementPosition(this.barStart).x + this.barMarginLeft - this.sliderWidth / 2;
        
        // Set original position
        this.Reposition();
    }
    
    // Provide the value to other JavaScripts
    TrackBar.prototype.GetValue = function()
    {
        return this.valueTextBox.value;
    }
    
    // Constructor

    // Initialised fields
    this.clientID = clientID; // Text ID of the parent control
    this.minimum = minimum; // The smallest value the control should contain
    this.maximum = maximum; // The largest value the control should contain
    this.smallChange = smallChange; // Minimum change that can be applied by the slider
    this.barPixelWidth = barPixelWidth; // Length of the actual bar in pixels

    // The input field of the ValueTextBox is calculated by predicting the client-side
    // DOM element name from the name of the parent control (passed as an argument)
    this.valueTextBox = document.getElementById(clientID + "_ValueTextBox");

    // Add the event that fires when the text is changed, so the slider
    // position can be updated to match.
    var changedDelegate = MakeDelegate(this, this.Reposition);

    if (document.addEventListener != null)
    { // e.g. Firefox
        this.valueTextBox.addEventListener("keyup", changedDelegate, true);
    }
    else
    { // e.g. IE
        this.valueTextBox.attachEvent("onkeyup", changedDelegate);
    }
    //findme
    
    // Set up the slider graphics
    this.barMarginLeft = 7; // pixels in left slider bar margin
    this.barMarginRight = 9; // pixels in left slider bar margin
    this.sliderWidth = 10; // 10 pixel slider
    this.barPieceWidth = 16; // pixels in each bar piece

    // Build the slider with DOM elements
    // It is easier to do this on the JavaScript side as adding controls to ASP.NET pages
    // at run time can be awkward.
    var sliderContainer = document.getElementById(clientID + "_sliderContainer"); // find the container DIV

    // Add the slider   
    this.slider = document.createElement("img");
    this.slider.src = "slideroff.gif";
    this.slider.className = "slider";
    this.slider.style.position = "absolute";
    sliderContainer.appendChild(this.slider);   

    // Add the bar start
    this.barStart = document.createElement("img");
    this.barStart.src = "barstart" + this.barPieceWidth + ".gif";
    sliderContainer.appendChild(this.barStart);   
        
    // Add the bar pieces
    var totalPieces = Math.floor((this.barPixelWidth + this.barMarginLeft + this.barMarginRight) / this.barPieceWidth);
    this.actualBarPixelWidth = totalPieces * this.barPieceWidth - (this.barMarginLeft + this.barMarginRight);
    
    var piecesToAdd = totalPieces - 2; // Two pieces already included (start and end)
    
    // Build up the main bar body
    for (var count = 0; count < piecesToAdd; count++)
    {
        var img = document.createElement("img");
        img.src = "barmid" + this.barPieceWidth + ".gif";
        sliderContainer.appendChild(img);   
    }

    // Add the bar end
    var img = document.createElement("img");
    img.src = "barend" + this.barPieceWidth + ".gif";
    sliderContainer.appendChild(img);   
    
    // Add a drag event on the slider button.
    // MakeDelegate is used to allow methods to be used as callbacks in events.
    this.slider.onmousedown = MakeDelegate(this, this.SliderDrag);

    // Add Load event. This is required to get the DOM positions as they are not
    // set up at this point.
    var loadDelegate = MakeDelegate(this, this.Load);

    if (document.addEventListener != null)
    { // e.g. Firefox
        window.addEventListener("load", loadDelegate, true);
    }
    else
    { // e.g. IE
        window.attachEvent("onload", loadDelegate);
    }
}
